﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using System.Linq;
using Newtonsoft.Json;
using UnityEngine.UI;
using TMPro;

namespace Mod
{
    public class Mod : MonoBehaviour
    {
        public static Texture2D Tongue = ModAPI.LoadTexture("Spont/SpiderTongue.png");
        public static AudioClip TongueIn = ModAPI.LoadSound("Spont/TongueIn.wav");
        public static AudioClip TongueOut = ModAPI.LoadSound("Spont/TongueOut.wav");
		public static Sprite[] SpontSkin = new Sprite[]
		{
			ModAPI.LoadSprite("Spont/SpiderHead.png"),
			ModAPI.LoadSprite("Spont/SpiderTorso.png"),
			ModAPI.LoadSprite("Spont/SpiderAbdomen.png"),
			ModAPI.LoadSprite("Spont/SpiderUpperLeg.png"),
			ModAPI.LoadSprite("Spont/SpiderLowerLeg.png"),
		};

		public static void Main()
        {
			ModAPI.Register( //Credits to Talon due to me just checking how he dealt with deleting limbs since last time I got a million errors.
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Human"),
					NameOverride = "Sponter",
					NameToOrderByOverride = "Spont",
					DescriptionOverride = "A weird spider-like human creature. Tends to stab other creatures with its <i>very long tongue</i>. (This can be disabled by breaking the upper torso)",
					CategoryOverride = ModAPI.FindCategory("Entities"),
					ThumbnailOverride = ModAPI.LoadSprite("SpontThumb.png"),
					AfterSpawn = (Instance) =>
					{
						PersonBehaviour Person = Instance.GetComponent<PersonBehaviour>();
						Person.CountDeathAsStat = false;
						foreach (var Limb in Person.Limbs)
						{
							if (Limb.gameObject.name.Contains("Foot"))
							{
								Limb.PhysicalBehaviour.Disintegrate();
								continue;
							}
							Limb.CirculationBehaviour.ForceSetAllLiquid(3f);
							Limb.CirculationBehaviour.BloodFlow += 2f;
							Limb.BloodMuscleStrengthRatio -= 0.3f;
							Limb.BaseStrength += 0.3f;
						}
						GameObject Head = Person.Limbs[0].gameObject;
						Head.GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[0];
						Head.GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderHeadFlesh.png"));
						Head.GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderHeadBone.png"));
						Head.transform.localPosition = new Vector2(-0.5909f, 0.202f);
						Head.GetComponent<SpriteRenderer>().sortingLayerName = "Background";
						Head.GetOrAddComponent<SponterLickingBehaviour>();

						var hjoint = Head.GetComponent<LimbBehaviour>().Joint;
						hjoint.anchor = new Vector2(-0.1393495f, -0.1903614f);
						hjoint.connectedAnchor = new Vector2(0.2471505f, 0.01163858f);
						var lim = hjoint.limits;
						lim.max = 40f;
						lim.min = -40f;
						hjoint.limits = lim;

						GameObject Torso1 = Person.Limbs[1].gameObject;
						Person.Limbs[1].CirculationBehaviour.IsPump = true;
						Person.Limbs[1].CirculationBehaviour.WasInitiallyPumping = true;
						Torso1.GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[1];
						Torso1.GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderTorsoFlesh.png"));
						Torso1.GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderTorsoBone.png"));
						Torso1.transform.localPosition = new Vector2(-0.9774f, 0f);
						GameObject Torso2 = Person.Limbs[2].gameObject;
						Person.Limbs[2].CirculationBehaviour.IsPump = true;
						Person.Limbs[2].CirculationBehaviour.WasInitiallyPumping = true;
						Torso2.GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[2];
						Torso2.GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderAbdomenFlesh.png"));
						Torso2.GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderAbdomenBone.png"));
						Torso2.transform.localPosition = new Vector2(-1.52f, 0f);

						GameObject.Destroy(Torso1.GetComponent<LimbBehaviour>().Joint);
						Torso1.GetComponent<LimbBehaviour>().HasJoint = false;

						var tjoint = Torso2.AddComponent<HingeJoint2D>();
						Torso2.GetComponent<LimbBehaviour>().Joint = tjoint;
						Torso2.GetComponent<LimbBehaviour>().HasJoint = true;
						tjoint.connectedBody = Torso1.GetComponent<Rigidbody2D>();
						tjoint.anchor = new Vector2(0.3142858f, 0f);
						tjoint.connectedAnchor = new Vector2(-0.2283142f, 0f);
						var lim1 = tjoint.limits;
						lim1.max = 80f;
						lim1.min = -80f;
						tjoint.limits = lim1;

						GameObject[] FrontLegs = new GameObject[]
						{
							Person.Limbs[4].gameObject,
							Person.Limbs[5].gameObject
						};
						FrontLegs[0].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[3];
						FrontLegs[0].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderUpperLegFlesh.png"));
						FrontLegs[0].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderUpperLegBone.png"));
						var ID = FrontLegs[0].GetComponent<SpriteRenderer>().sortingLayerID;
						FrontLegs[0].transform.localPosition = new Vector2(-1.727f, -0.125f);
						FrontLegs[1].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[4];
						FrontLegs[1].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderLowerLegFlesh.png"));
						FrontLegs[1].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderLowerLegBone.png"));
						FrontLegs[1].transform.localPosition = new Vector2(-1.727f, -0.539f);

						GameObject[] BackLegs = new GameObject[]
						{
							Person.Limbs[7].gameObject,
							Person.Limbs[8].gameObject
						};
						BackLegs[0].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[3];
						BackLegs[0].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderUpperLegFlesh.png"));
						BackLegs[0].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderUpperLegBone.png"));
						BackLegs[0].GetComponent<SpriteRenderer>().sortingLayerID = ID;
						BackLegs[0].transform.localPosition = new Vector2(-1.5341f, -0.125f);
						BackLegs[0].GetComponent<LimbBehaviour>().ConnectedLimbs.Add(Torso2.GetComponent<LimbBehaviour>());
						BackLegs[1].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[4];
						BackLegs[1].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderLowerLegFlesh.png"));
						BackLegs[1].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderLowerLegBone.png"));
						BackLegs[1].GetComponent<SpriteRenderer>().sortingLayerID = ID;
						BackLegs[1].transform.localPosition = new Vector2(-1.5341f, -0.539f);

						Torso1.GetComponent<LimbBehaviour>().IsLethalToBreak = false;
						Torso1.GetComponent<LimbBehaviour>().RoughClassification = LimbBehaviour.BodyPart.Head;

						var newPushes = Torso2.GetComponent<CirculationBehaviour>().PushesTo.ToList();
						newPushes.Add(FrontLegs[0].GetComponent<CirculationBehaviour>());
						newPushes.Add(BackLegs[0].GetComponent<CirculationBehaviour>());
						Torso2.GetComponent<CirculationBehaviour>().PushesTo = newPushes.ToArray();
						//Torso2.GetComponent<ConnectedNodeBehaviour>().IsRoot = true;
						//Torso2.GetComponent<ConnectedNodeBehaviour>().RootPropagation();
						FrontLegs[0].GetComponent<CirculationBehaviour>().Source = Torso2.GetComponent<CirculationBehaviour>();
						BackLegs[0].GetComponent<CirculationBehaviour>().Source = Torso2.GetComponent<CirculationBehaviour>();

						Torso2.GetComponent<ConnectedNodeBehaviour>().Connections = new ConnectedNodeBehaviour[]
						{
							FrontLegs[0].GetComponent<ConnectedNodeBehaviour>(),
							BackLegs[0].GetComponent<ConnectedNodeBehaviour>(),
							Torso1.GetComponent<ConnectedNodeBehaviour>()
						};

						FrontLegs[0].GetComponent<ConnectedNodeBehaviour>().Connections = new ConnectedNodeBehaviour[]
						{
							FrontLegs[1].GetComponent<ConnectedNodeBehaviour>(),
							Torso2.GetComponent<ConnectedNodeBehaviour>()
						};

						BackLegs[0].GetComponent<ConnectedNodeBehaviour>().Connections = new ConnectedNodeBehaviour[]
						{
							BackLegs[1].GetComponent<ConnectedNodeBehaviour>(),
							Torso2.GetComponent<ConnectedNodeBehaviour>()
						};

						FrontLegs[0].GetComponent<LimbBehaviour>().ConnectedLimbs = new List<LimbBehaviour>()
						{
							FrontLegs[1].GetComponent<LimbBehaviour>(),
							Torso2.GetComponent<LimbBehaviour>()
						};

						BackLegs[0].GetComponent<LimbBehaviour>().ConnectedLimbs = new List<LimbBehaviour>()
						{
							BackLegs[1].GetComponent<LimbBehaviour>(),
							Torso2.GetComponent<LimbBehaviour>()
						};

						FrontLegs[0].GetComponent<SkinMaterialHandler>().adjacentLimbs = new SkinMaterialHandler[]
						{
							FrontLegs[1].GetComponent<SkinMaterialHandler>(),
							Torso2.GetComponent<SkinMaterialHandler>()
						};

						BackLegs[0].GetComponent<SkinMaterialHandler>().adjacentLimbs = new SkinMaterialHandler[]
						{
							BackLegs[1].GetComponent<SkinMaterialHandler>(),
							Torso2.GetComponent<SkinMaterialHandler>()
						};

						Torso2.GetComponent<SkinMaterialHandler>().adjacentLimbs = new SkinMaterialHandler[]
						{
							BackLegs[0].GetComponent<SkinMaterialHandler>(),
							FrontLegs[0].GetComponent<SkinMaterialHandler>()
						};

						Torso2.GetComponent<LimbBehaviour>().ConnectedLimbs = new List<LimbBehaviour>()
						{
							BackLegs[0].GetComponent<LimbBehaviour>(),
							FrontLegs[0].GetComponent<LimbBehaviour>()
						};

						GameObject[] FrontArms = new GameObject[]
						{
							Person.Limbs[10].gameObject,
							Person.Limbs[12].gameObject
						};
						Instance.transform.Find("FrontArm").localPosition = Instance.transform.Find("FrontLeg").localPosition;
						Instance.transform.Find("BackArm").localPosition = Instance.transform.Find("FrontLeg").localPosition;
						FrontArms[0].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[3];
						FrontArms[0].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderUpperLegFlesh.png"));
						FrontArms[0].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderUpperLegBone.png"));
						FrontArms[0].GetComponent<SpriteRenderer>().sortingLayerID = ID;
						FrontArms[0].transform.localPosition = new Vector2(-1.3343f, -0.125f);
						FrontArms[1].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[4];
						FrontArms[1].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderLowerLegFlesh.png"));
						FrontArms[1].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderLowerLegBone.png"));
						FrontArms[1].GetComponent<SpriteRenderer>().sortingLayerID = ID;
						FrontArms[1].transform.localPosition = new Vector2(-1.3343f, -0.539f);

						GameObject[] BackArms = new GameObject[]
						{
							Person.Limbs[11].gameObject,
							Person.Limbs[13].gameObject
						};
						BackArms[0].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[3];
						BackArms[0].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderUpperLegFlesh.png"));
						BackArms[0].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderUpperLegBone.png"));
						BackArms[0].GetComponent<SpriteRenderer>().sortingLayerID = ID;
						BackArms[0].transform.localPosition = new Vector2(-1.1348f, -0.127f); //+0.002
						BackArms[1].GetComponent<SpriteRenderer>().sprite = Mod.SpontSkin[4];
						BackArms[1].GetComponent<SpriteRenderer>().material.SetTexture("_FleshTex", ModAPI.LoadTexture("Spont/SpiderLowerLegFlesh.png"));
						BackArms[1].GetComponent<SpriteRenderer>().material.SetTexture("_BoneTex", ModAPI.LoadTexture("Spont/SpiderLowerLegBone.png"));
						BackArms[1].GetComponent<SpriteRenderer>().sortingLayerID = ID;
						BackArms[1].transform.localPosition = new Vector2(-1.1348f, -0.542f);

						var legjoint1 = FrontLegs[0].GetComponent<LimbBehaviour>().Joint;
						legjoint1.anchor = new Vector2(-0.001070976f, 0.1242427f);
						legjoint1.connectedAnchor = new Vector2(-0.208071f, -0.0007572919f);
						legjoint1.connectedBody = Torso2.GetComponent<Rigidbody2D>();
						var lim2 = legjoint1.limits;
						lim2.max = 359f;
						lim2.min = -90f;
						legjoint1.limits = lim2;

						var legjoint2 = FrontLegs[1].GetComponent<LimbBehaviour>().Joint;
						legjoint2.anchor = new Vector2(-0.0003910065f, 0.2414933f);
						legjoint2.connectedAnchor = new Vector2(-0.0003910065f, -0.1725067f);
						var lim3 = legjoint2.limits;
						lim3.max = 90f;
						lim3.min = -90f;
						legjoint2.limits = lim3;

						var legjoint3 = BackLegs[0].GetComponent<LimbBehaviour>().Joint;
						legjoint3.anchor = new Vector2(-0.001070976f, 0.1242427f);
						legjoint3.connectedAnchor = new Vector2(-0.01517105f, -0.0007572919f);
						legjoint3.connectedBody = Torso2.GetComponent<Rigidbody2D>();
						var lim4 = legjoint3.limits;
						lim4.max = 359f;
						lim4.min = -90f;
						legjoint3.limits = lim4;

						var legjoint4 = BackLegs[1].GetComponent<LimbBehaviour>().Joint;
						legjoint4.anchor = new Vector2(-0.0003910065f, 0.2414933f);
						legjoint4.connectedAnchor = new Vector2(-0.0003910065f, -0.1725067f);
						var lim5 = legjoint4.limits;
						lim5.max = 90f;
						lim5.min = -90f;
						legjoint4.limits = lim5;

						var legjoint5 = FrontArms[0].GetComponent<LimbBehaviour>().Joint;
						legjoint5.anchor = new Vector2(-0.001070976f, 0.1242427f);
						legjoint5.connectedAnchor = new Vector2(0.184629f, -0.0007572919f);
						legjoint5.connectedBody = Torso2.GetComponent<Rigidbody2D>();
						var lim6 = legjoint5.limits;
						lim6.max = 359f;
						lim6.min = -90f;
						legjoint5.limits = lim6;

						var legjoint6 = FrontArms[1].GetComponent<LimbBehaviour>().Joint;
						legjoint6.anchor = new Vector2(-0.0003910065f, 0.2414933f);
						legjoint6.connectedAnchor = new Vector2(-0.0003910065f, -0.1725067f);
						legjoint6.connectedBody = FrontArms[0].GetComponent<Rigidbody2D>();
						var lim7 = legjoint6.limits;
						lim7.max = 90f;
						lim7.min = -90f;
						legjoint6.limits = lim7;

						var legjoint7 = BackArms[0].GetComponent<LimbBehaviour>().Joint;
						legjoint7.anchor = new Vector2(-0.001070976f, 0.1242427f);
						legjoint7.connectedAnchor = new Vector2(-0.01517105f, -0.0007572919f);
						legjoint7.connectedBody = Torso1.GetComponent<Rigidbody2D>();
						var lim8 = legjoint7.limits;
						lim8.max = 359f;
						lim8.min = -90f;
						legjoint7.limits = lim8;

						var legjoint8 = BackArms[1].GetComponent<LimbBehaviour>().Joint;
						legjoint8.anchor = new Vector2(-0.0003910065f, 0.2414933f);
						legjoint8.connectedAnchor = new Vector2(-0.0003910065f, -0.1725067f);
						legjoint8.connectedBody = BackArms[0].GetComponent<Rigidbody2D>();
						var lim9 = legjoint8.limits;
						lim9.max = 90f;
						lim9.min = -90f;
						legjoint8.limits = lim9;
						Person.Limbs[3].PhysicalBehaviour.Disintegrate();
						foreach (var Limb in Person.Limbs)
						{
							Limb.gameObject.FixColliders();
						}
						var nocol = Instance.AddComponent<NoCollide>();
						nocol.NoCollideSetA = Instance.GetComponentsInChildren<Collider2D>();
						nocol.NoCollideSetB = Instance.GetComponentsInChildren<Collider2D>();
						///////////////////////////////////////////////////////////////////// ANIMATIONS
						var StandPose = Person.ActivePose;
						StandPose.Angles = new List<RagdollPose.LimbPose>();
						foreach (var limb in Person.Limbs)
						{
							if (limb.gameObject.name == "MiddleTorso" || limb.gameObject.name == "Head")
							{
								var animstuff = new RagdollPose.LimbPose(limb, -15f);
								StandPose.Angles.Add(animstuff);
							}
							else
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								StandPose.Angles.Add(animstuff);
							}
						}
						StandPose.ConstructDictionary();

						var SitPose = Person.LinkedPoses[PoseState.Sitting];
						SitPose.Angles = new List<RagdollPose.LimbPose>();
						foreach (var limb in Person.Limbs)
						{
							if (limb.gameObject.name.Contains("Upper"))
							{
								var animstuff = new RagdollPose.LimbPose(limb, -90f);
								SitPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name.Contains("Lower"))
							{
								var animstuff = new RagdollPose.LimbPose(limb, 30f);
								SitPose.Angles.Add(animstuff);
							}
							else
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								SitPose.Angles.Add(animstuff);
							}
						}
						SitPose.ConstructDictionary();

						var WalkPose = Person.LinkedPoses[PoseState.Walking];
						WalkPose.Angles = new List<RagdollPose.LimbPose>();
						WalkPose.AnimationSpeedMultiplier = 1.5f;
						WalkPose.Rigidity += 0.7f;
						foreach (var limb in Person.Limbs)
						{
							if (limb.gameObject.name == "UpperArm")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = 75f;
								animstuff.EndAngle = -5f;
								animstuff.TimeOffset = 0f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "LowerArm")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = 75f;
								animstuff.EndAngle = -5f;
								animstuff.TimeOffset = 0f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "UpperArmFront")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = -75f;
								animstuff.EndAngle = 5f;
								animstuff.TimeOffset = -0.2f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "LowerArmFront")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = -75f;
								animstuff.EndAngle = 5f;
								animstuff.TimeOffset = -0.2f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "UpperLeg")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = -75f;
								animstuff.EndAngle = 5f;
								animstuff.TimeOffset = -0.4f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "LowerLeg")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = 75f;
								animstuff.EndAngle = -5f;
								animstuff.TimeOffset = -0.4f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "UpperLegFront")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = -75f;
								animstuff.EndAngle = 5f;
								animstuff.TimeOffset = -0.6f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "LowerLegFront")
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								animstuff.Animated = true;
								animstuff.StartAngle = 75f;
								animstuff.EndAngle = -5f;
								animstuff.TimeOffset = -0.6f;
								animstuff.AnimationDuration = 0.5f;
								WalkPose.Angles.Add(animstuff);
							}
							else
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								WalkPose.Angles.Add(animstuff);
							}
						}
						WalkPose.ConstructDictionary();

						var CowerPose = Person.LinkedPoses[PoseState.Protective];
						CowerPose.Angles = new List<RagdollPose.LimbPose>();
						foreach (var limb in Person.Limbs)
						{
							if (limb.gameObject.name.Contains("UpperArmFront"))
							{
								var animstuff = new RagdollPose.LimbPose(limb, -90f);
								CowerPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name.Contains("UpperArm"))
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								CowerPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name.Contains("LowerArmFront"))
							{
								var animstuff = new RagdollPose.LimbPose(limb, -90f);
								CowerPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name.Contains("UpperLeg"))
							{
								var animstuff = new RagdollPose.LimbPose(limb, -90f);
								CowerPose.Angles.Add(animstuff);
							}
							else if (limb.gameObject.name == "Head")
							{
								var animstuff = new RagdollPose.LimbPose(limb, -45f);
								CowerPose.Angles.Add(animstuff);
							}
							else
							{
								var animstuff = new RagdollPose.LimbPose(limb, 0f);
								CowerPose.Angles.Add(animstuff);
							}
						}
						CowerPose.ConstructDictionary();
					}
				}
			);

		}
    }

	public class SponterLickingBehaviour : MonoBehaviour
	{
		private LineRenderer Tongue;
		private Transform LineEnd;
		private RaycastHit2D[] losBuffer = new RaycastHit2D[5];
		private bool IsAttacking;
		private bool Idle;
		private AudioSource Aud;
		private LimbBehaviour Limb;

		public bool Disabled
		{
			get
			{
				return gameObject.GetComponent<LimbBehaviour>().Person.Limbs[1].Broken;
			}
		}

		private void Start()
		{
			Aud = gameObject.AddComponent<AudioSource>();
			gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
			Tongue = gameObject.AddComponent<LineRenderer>();
			Tongue.material = UnityEngine.Object.Instantiate<Material>(ModAPI.FindMaterial("Sprites-Default"));
			Tongue.material.mainTexture = Mod.Tongue;
			Tongue.widthMultiplier = 0.04f;
			Tongue.sortingLayerName = "Bottom";
			Tongue.enabled = false;
			GameObject End = new GameObject("LineEnd");
			End.transform.SetParent(transform);
			End.transform.localPosition = Vector2.zero;
			End.transform.localScale = Vector2.zero;
			var rig = End.AddComponent<Rigidbody2D>();
			rig.mass = 0f;
			rig.bodyType = RigidbodyType2D.Kinematic;
			LineEnd = End.transform;
			Tongue.positionCount = 2;
			Tongue.SetPosition(0, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f)));
		}

		private void Update()
		{
			if (IsAttacking)
			{
				Tongue.SetPosition(0, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f)));
			}
			if (Idle)
			{
				Tongue.SetPosition(1, Limb.transform.position);
			}
		}

		private void FixedUpdate()
		{
			Collider2D[] Results = new Collider2D[30];
			if (IsAttacking || UnityEngine.Random.value < 0.975f || !gameObject.GetComponent<LimbBehaviour>().IsConsideredAlive || gameObject.GetComponent<LimbBehaviour>().Person.Consciousness < 0.9f || Disabled)
			{
				return;
			}
			int cols = Physics2D.OverlapCircleNonAlloc(transform.position, 5f, Results, LayerMask.GetMask(new string[] { "Objects", "Bounds", "CollidingDebris" }));
			for (var i = 0; i < cols; i++)
			{
				if (Results[i].gameObject.TryGetComponent<LimbBehaviour>(out Limb) && Limb.RoughClassification == LimbBehaviour.BodyPart.Head && Limb.transform.root != transform.root && Limb.IsConsideredAlive)
				{
					if (HasLineOfSight(Limb))
					{
						StartCoroutine(Attack(Limb));
						return;
					}
				}
			}
		}

		private bool HasLineOfSight(LimbBehaviour Limb)
		{
			int num = Physics2D.LinecastNonAlloc(transform.position, Limb.transform.position, losBuffer, LayerMask.GetMask(new string[] { "Objects", "Bounds", "CollidingDebris" }));
			if (num == 0)
			{
				return false;
			}
			for (int i = 0; i < num; i++)
			{
				RaycastHit2D raycastHit2D = losBuffer[i];
				if (raycastHit2D.transform.root != Limb.transform.root && raycastHit2D.transform.root != transform.root)
				{
					return false;
				}
			}
			return true;
		}

		private IEnumerator Attack(LimbBehaviour Limb)
		{
			ModAPI.Notify("Seen!");
			Debug.Log(IsAttacking.ToString());
			IsAttacking = true;
			Tongue.enabled = true;
			Aud.PlayOneShot(Mod.TongueOut);
			while (Limb != null && Vector2.Distance(LineEnd.transform.position, Limb.transform.position) > 0.15f)
			{
				if (Limb != null)
				{
					LineEnd.transform.position = Vector3.Lerp(LineEnd.transform.position, Limb.transform.position, Time.deltaTime * 10f);
					Tongue.SetPosition(1, LineEnd.transform.position);
				}
				else
				{
					IsAttacking = false;
					Tongue.SetPosition(1, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f)));
					Tongue.enabled = false;
					Debug.Log("Gone?");
					yield break;
				}
				yield return new WaitForEndOfFrame();
			}
			if (Limb == null)
            {
				IsAttacking = false;
				Tongue.SetPosition(1, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f)));
				Tongue.enabled = false;
				Debug.Log("Gone?");
				yield break;
			}
			var Joint = Limb.gameObject.AddComponent<FrictionJoint2D>();
			Joint.autoConfigureConnectedAnchor = true;
			Joint.enableCollision = false;
			Joint.maxForce = 1f;
			Joint.connectedBody = LineEnd.gameObject.GetComponent<Rigidbody2D>();
			Joint.maxTorque = 1f;
			Joint.anchor = LineEnd.InverseTransformPoint(LineEnd.transform.position);
			Limb.Damage(0.3f);
			Limb.Health -= 0.3f;
			Limb.SkinMaterialHandler.AddDamagePoint(DamageType.Bullet, LineEnd.transform.position, 50f);
			Limb.PhysicalBehaviour.PlayClipOnce(Limb.PhysicalBehaviour.Properties.StabSound.PickRandom<AudioClip>(), 0.5f);
			Idle = true;
			yield return new WaitForSeconds(0.5f);
			Idle = false;
			if (Limb != null)
			{
				Limb.CirculationBehaviour.Cut(LineEnd.transform.position, transform.position - Limb.transform.position);
				Limb.PhysicalBehaviour.PlayClipOnce(Limb.PhysicalBehaviour.Properties.StabSound.PickRandom<AudioClip>(), 1.5f);
				Limb.SkinMaterialHandler.AddDamagePoint(DamageType.Bullet, LineEnd.transform.position, 150f);
				Limb.Damage(150f);
				GameObject.Destroy(Joint);
				Limb.PhysicalBehaviour.rigidbody.AddForce((transform.position - Limb.transform.position), ForceMode2D.Impulse);
				Aud.PlayOneShot(Mod.TongueIn);
			}
			else
			{
				IsAttacking = false;
				Tongue.SetPosition(1, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f)));
				Tongue.enabled = false;
				Debug.Log("Gone?");
				yield break;
			}
			while (Vector2.Distance(LineEnd.transform.position, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f))) > 0.15f)
			{
				LineEnd.transform.position = Vector3.Lerp(LineEnd.transform.position, transform.TransformPoint(new Vector3(0.12f, -0.04f, 0f)), Time.deltaTime * 10f);
				Tongue.SetPosition(1, LineEnd.transform.position);
				yield return new WaitForEndOfFrame();
			}
			IsAttacking = false;
			Limb = null;
			Tongue.SetPosition(1, transform.TransformPoint(new Vector3(0.13f, 0.02f, 0f)));
			Tongue.enabled = false;
			yield return new WaitForEndOfFrame();
		}
	}
}
